@@ -22,9 +22,9 @@
 
 /****************************************************************************
 
- Copyright (C) 2002-2013 Gilles Debunne. All rights reserved.
+ Copyright (C) 2002-2014 Gilles Debunne. All rights reserved.
 
- This file is part of the QGLViewer library version 2.4.0.
+ This file is part of the QGLViewer library version 2.6.4.
 
  http://www.libqglviewer.com - contact@libqglviewer.com
 
@@ -46,13 +46,8 @@
 #include "Exporter.h"
 #include "../qglviewer.h"
 
-#if QT_VERSION >= 0x040000
-# include <QFile>
-# include <QMessageBox>
-#else
-# include <qfile.h>
-# include <qmessagebox.h>
-#endif
+#include <QFile>
+#include <QMessageBox>
 
 using namespace vrender ;
 using namespace std ;
@@ -69,11 +64,7 @@ void Exporter::exportToFile(const QString& filename,
 {
 	QFile file(filename);
 
-#if QT_VERSION >= 0x040000
 	if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
-#else
-	if (!file.open(IO_WriteOnly | IO_Translate)) {
-#endif
 		QMessageBox::warning(NULL, QGLViewer::tr("Exporter error", "Message box window title"), QGLViewer::tr("Unable to open file %1.").arg(filename));
 		return;
 	}
@@ -82,7 +73,7 @@ void Exporter::exportToFile(const QString& filename,
 
 	writeHeader(out) ;
 
-        unsigned int N = primitive_tab.size()/200 + 1 ;
+		unsigned int N = primitive_tab.size()/200 + 1 ;
 
 	for(unsigned int i=0;i<primitive_tab.size();++i)
 	{
