@@ -22,9 +22,9 @@
 
 /****************************************************************************
 
- Copyright (C) 2002-2013 Gilles Debunne. All rights reserved.
+ Copyright (C) 2002-2014 Gilles Debunne. All rights reserved.
 
- This file is part of the QGLViewer library version 2.4.0.
+ This file is part of the QGLViewer library version 2.6.4.
 
  http://www.libqglviewer.com - contact@libqglviewer.com
 
@@ -50,13 +50,8 @@
 #include "Primitive.h"
 
 #include "../config.h"
-#if QT_VERSION >= 0x040000
-# include <QTextStream>
-# include <QString>
-#else
-# include <qtextstream.h>
-# include <qstring.h>
-#endif
+#include <QTextStream>
+#include <QString>
 
 namespace vrender
 {
