@@ -22,9 +22,9 @@
 
 /****************************************************************************
 
- Copyright (C) 2002-2013 Gilles Debunne. All rights reserved.
+ Copyright (C) 2002-2014 Gilles Debunne. All rights reserved.
 
- This file is part of the QGLViewer library version 2.4.0.
+ This file is part of the QGLViewer library version 2.6.4.
 
  http://www.libqglviewer.com - contact@libqglviewer.com
 
@@ -47,8 +47,8 @@
 
 Project:   Generic Polygon Clipper
 
-           A new algorithm for calculating the difference, intersection,
-           exclusive-or or union of arbitrary polygon sets.
+		   A new algorithm for calculating the difference, intersection,
+		   exclusive-or or union of arbitrary polygon sets.
 
 File:      gpc.h
 Author:    Alan Murta (email: gpc@cs.man.ac.uk)
@@ -56,20 +56,20 @@ Version:   2.32
 Date:      17th December 2004
 
 Copyright: (C) 1997-2004, Advanced Interfaces Group,
-           University of Manchester.
+		   University of Manchester.
 
-           This software is free for non-commercial use. It may be copied,
-           modified, and redistributed provided that this copyright notice
-           is preserved on all copies. The intellectual property rights of
-           the algorithms used reside with the University of Manchester
-           Advanced Interfaces Group.
+		   This software is free for non-commercial use. It may be copied,
+		   modified, and redistributed provided that this copyright notice
+		   is preserved on all copies. The intellectual property rights of
+		   the algorithms used reside with the University of Manchester
+		   Advanced Interfaces Group.
 
-           You may not use this software, in whole or in part, in support
-           of any commercial product without the express consent of the
-           author.
+		   You may not use this software, in whole or in part, in support
+		   of any commercial product without the express consent of the
+		   author.
 
-           There is no warranty or other guarantee of fitness of this
-           software for any purpose. It is provided solely "as is".
+		   There is no warranty or other guarantee of fitness of this
+		   software for any purpose. It is provided solely "as is".
 
 ===========================================================================
 */
@@ -82,7 +82,7 @@ Copyright: (C) 1997-2004, Advanced Interfaces Group,
 
 /*
 ===========================================================================
-                               Constants
+							   Constants
 ===========================================================================
 */
 
@@ -96,7 +96,7 @@ Copyright: (C) 1997-2004, Advanced Interfaces Group,
 
 /*
 ===========================================================================
-                           Public Data Types
+						   Public Data Types
 ===========================================================================
 */
 
@@ -116,54 +116,54 @@ typedef struct                      /* Polygon vertex structure          */
 
 typedef struct                      /* Vertex list structure             */
 {
-  int                 num_vertices; /* Number of vertices in list        */
+  long                num_vertices; /* Number of vertices in list        */
   gpc_vertex         *vertex;       /* Vertex array pointer              */
 } gpc_vertex_list;
 
 typedef struct                      /* Polygon set structure             */
 {
-  int                 num_contours; /* Number of contours in polygon     */
+  unsigned long       num_contours; /* Number of contours in polygon     */
   int                *hole;         /* Hole / external contour flags     */
   gpc_vertex_list    *contour;      /* Contour array pointer             */
 } gpc_polygon;
 
 typedef struct                      /* Tristrip set structure            */
 {
-  int                 num_strips;   /* Number of tristrips               */
+  unsigned long       num_strips;   /* Number of tristrips               */
   gpc_vertex_list    *strip;        /* Tristrip array pointer            */
 } gpc_tristrip;
 
 
 /*
 ===========================================================================
-                       Public Function Prototypes
+					   Public Function Prototypes
 ===========================================================================
 */
 
 void gpc_read_polygon        (FILE            *infile_ptr,
-                              int              read_hole_flags,
-                              gpc_polygon     *polygon);
+							  int              read_hole_flags,
+							  gpc_polygon     *polygon);
 
 void gpc_write_polygon       (FILE            *outfile_ptr,
-                              int              write_hole_flags,
-                              gpc_polygon     *polygon);
+							  int              write_hole_flags,
+							  gpc_polygon     *polygon);
 
 void gpc_add_contour         (gpc_polygon     *polygon,
-                              gpc_vertex_list *contour,
-                              int              hole);
+							  gpc_vertex_list *contour,
+							  int              hole);
 
 void gpc_polygon_clip        (gpc_op           set_operation,
-                              gpc_polygon     *subject_polygon,
-                              gpc_polygon     *clip_polygon,
-                              gpc_polygon     *result_polygon);
+							  gpc_polygon     *subject_polygon,
+							  gpc_polygon     *clip_polygon,
+							  gpc_polygon     *result_polygon);
 
 void gpc_tristrip_clip       (gpc_op           set_operation,
-                              gpc_polygon     *subject_polygon,
-                              gpc_polygon     *clip_polygon,
-                              gpc_tristrip    *result_tristrip);
+							  gpc_polygon     *subject_polygon,
+							  gpc_polygon     *clip_polygon,
+							  gpc_tristrip    *result_tristrip);
 
 void gpc_polygon_to_tristrip (gpc_polygon     *polygon,
-                              gpc_tristrip    *tristrip);
+							  gpc_tristrip    *tristrip);
 
 void gpc_free_polygon        (gpc_polygon     *polygon);
 
@@ -174,6 +174,6 @@ void gpc_free_tristrip       (gpc_tristrip    *tristrip);
 
 /*
 ===========================================================================
-                           End of file: gpc.h
+						   End of file: gpc.h
 ===========================================================================
 */
