@@ -1,8 +1,8 @@
 /****************************************************************************
 
- Copyright (C) 2002-2013 Gilles Debunne. All rights reserved.
+ Copyright (C) 2002-2014 Gilles Debunne. All rights reserved.
 
- This file is part of the QGLViewer library version 2.4.0.
+ This file is part of the QGLViewer library version 2.6.4.
 
  http://www.libqglviewer.com - contact@libqglviewer.com
 
@@ -23,6 +23,7 @@
 #include "constraint.h"
 #include "frame.h"
 #include "camera.h"
+#include "manipulatedCameraFrame.h"
 
 using namespace qglviewer;
 using namespace std;
@@ -36,55 +37,55 @@ using namespace std;
 translationConstraintType() and rotationConstraintType() are set to AxisPlaneConstraint::FREE.
 translationConstraintDirection() and rotationConstraintDirection() are set to (0,0,0). */
 AxisPlaneConstraint::AxisPlaneConstraint()
-  : translationConstraintType_(FREE), rotationConstraintType_(FREE)
+	: translationConstraintType_(FREE), rotationConstraintType_(FREE)
 {
-  // Do not use set since setRotationConstraintType needs a read.
+	// Do not use set since setRotationConstraintType needs a read.
 }
 
 /*! Simply calls setTranslationConstraintType() and setTranslationConstraintDirection(). */
 void AxisPlaneConstraint::setTranslationConstraint(Type type, const Vec& direction)
 {
-  setTranslationConstraintType(type);
-  setTranslationConstraintDirection(direction);
+	setTranslationConstraintType(type);
+	setTranslationConstraintDirection(direction);
 }
 
 /*! Defines the translationConstraintDirection(). The coordinate system where \p direction is expressed depends on your class implementation. */
 void AxisPlaneConstraint::setTranslationConstraintDirection(const Vec& direction)
 {
-  if ((translationConstraintType()!=AxisPlaneConstraint::FREE) && (translationConstraintType()!=AxisPlaneConstraint::FORBIDDEN))
-    {
-      const float norm = direction.norm();
-      if (norm < 1E-8)
+	if ((translationConstraintType()!=AxisPlaneConstraint::FREE) && (translationConstraintType()!=AxisPlaneConstraint::FORBIDDEN))
 	{
-	  qWarning("AxisPlaneConstraint::setTranslationConstraintDir: null vector for translation constraint");
-	  translationConstraintType_ = AxisPlaneConstraint::FREE;
+		const qreal norm = direction.norm();
+		if (norm < 1E-8)
+		{
+			qWarning("AxisPlaneConstraint::setTranslationConstraintDir: null vector for translation constraint");
+			translationConstraintType_ = AxisPlaneConstraint::FREE;
+		}
+		else
+			translationConstraintDir_ = direction/norm;
 	}
-      else
-	translationConstraintDir_ = direction/norm;
-    }
 }
 
 /*! Simply calls setRotationConstraintType() and setRotationConstraintDirection(). */
 void AxisPlaneConstraint::setRotationConstraint(Type type, const Vec& direction)
 {
-  setRotationConstraintType(type);
-  setRotationConstraintDirection(direction);
+	setRotationConstraintType(type);
+	setRotationConstraintDirection(direction);
 }
 
 /*! Defines the rotationConstraintDirection(). The coordinate system where \p direction is expressed depends on your class implementation. */
 void AxisPlaneConstraint::setRotationConstraintDirection(const Vec& direction)
 {
-  if ((rotationConstraintType()!=AxisPlaneConstraint::FREE) && (rotationConstraintType()!=AxisPlaneConstraint::FORBIDDEN))
-    {
-      float norm = direction.norm();
-      if (norm < 1E-8)
+	if ((rotationConstraintType()!=AxisPlaneConstraint::FREE) && (rotationConstraintType()!=AxisPlaneConstraint::FORBIDDEN))
 	{
-	  qWarning("AxisPlaneConstraint::setRotationConstraintDir: null vector for rotation constraint");
-	  rotationConstraintType_ = AxisPlaneConstraint::FREE;
+		const qreal norm = direction.norm();
+		if (norm < 1E-8)
+		{
+			qWarning("AxisPlaneConstraint::setRotationConstraintDir: null vector for rotation constraint");
+			rotationConstraintType_ = AxisPlaneConstraint::FREE;
+		}
+		else
+			rotationConstraintDir_ = direction/norm;
 	}
-      else
-	rotationConstraintDir_ = direction/norm;
-    }
 }
 
 /*! Set the Type() of the rotationConstraintType(). Default is AxisPlaneConstraint::FREE.
@@ -100,13 +101,13 @@ void AxisPlaneConstraint::setRotationConstraintDirection(const Vec& direction)
  will be ignored. */
 void AxisPlaneConstraint::setRotationConstraintType(Type type)
 {
-  if (rotationConstraintType() == AxisPlaneConstraint::PLANE)
-    {
-      qWarning("AxisPlaneConstraint::setRotationConstraintType: the PLANE type cannot be used for a rotation constraints");
-      return;
-    }
+	if (rotationConstraintType() == AxisPlaneConstraint::PLANE)
+	{
+		qWarning("AxisPlaneConstraint::setRotationConstraintType: the PLANE type cannot be used for a rotation constraints");
+		return;
+	}
 
-  rotationConstraintType_ = type;
+	rotationConstraintType_ = type;
 }
 
 
@@ -119,23 +120,23 @@ void AxisPlaneConstraint::setRotationConstraintType(Type type)
   translationConstraintDirection(). */
 void LocalConstraint::constrainTranslation(Vec& translation, Frame* const frame)
 {
-  Vec proj;
-  switch (translationConstraintType())
-    {
-    case AxisPlaneConstraint::FREE:
-      break;
-    case AxisPlaneConstraint::PLANE:
-      proj = frame->rotation().rotate(translationConstraintDirection());
-      translation.projectOnPlane(proj);
-      break;
-    case AxisPlaneConstraint::AXIS:
-      proj = frame->rotation().rotate(translationConstraintDirection());
-      translation.projectOnAxis(proj);
-      break;
-    case AxisPlaneConstraint::FORBIDDEN:
-      translation = Vec(0.0, 0.0, 0.0);
-      break;
-    }
+	Vec proj;
+	switch (translationConstraintType())
+	{
+	case AxisPlaneConstraint::FREE:
+		break;
+	case AxisPlaneConstraint::PLANE:
+		proj = frame->rotation().rotate(translationConstraintDirection());
+		translation.projectOnPlane(proj);
+		break;
+	case AxisPlaneConstraint::AXIS:
+		proj = frame->rotation().rotate(translationConstraintDirection());
+		translation.projectOnAxis(proj);
+		break;
+	case AxisPlaneConstraint::FORBIDDEN:
+		translation = Vec(0.0, 0.0, 0.0);
+		break;
+	}
 }
 
 /*! When rotationConstraintType() is AxisPlaneConstraint::AXIS, constrain \p rotation to be a rotation
@@ -143,24 +144,24 @@ void LocalConstraint::constrainTranslation(Vec& translation, Frame* const frame)
   rotationConstraintDirection(). */
 void LocalConstraint::constrainRotation(Quaternion& rotation, Frame* const)
 {
-  switch (rotationConstraintType())
-    {
-    case AxisPlaneConstraint::FREE:
-      break;
-    case AxisPlaneConstraint::PLANE:
-      break;
-    case AxisPlaneConstraint::AXIS:
-      {
-	Vec axis = rotationConstraintDirection();
-	Vec quat = Vec(rotation[0], rotation[1], rotation[2]);
-	quat.projectOnAxis(axis);
-	rotation = Quaternion(quat, 2.0*acos(rotation[3]));
-      }
-      break;
-    case AxisPlaneConstraint::FORBIDDEN:
-      rotation = Quaternion(); // identity
-      break;
-    }
+	switch (rotationConstraintType())
+	{
+	case AxisPlaneConstraint::FREE:
+		break;
+	case AxisPlaneConstraint::PLANE:
+		break;
+	case AxisPlaneConstraint::AXIS:
+	{
+		Vec axis = rotationConstraintDirection();
+		Vec quat = Vec(rotation[0], rotation[1], rotation[2]);
+		quat.projectOnAxis(axis);
+		rotation = Quaternion(quat, 2.0*acos(rotation[3]));
+	}
+		break;
+	case AxisPlaneConstraint::FORBIDDEN:
+		rotation = Quaternion(); // identity
+		break;
+	}
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -172,33 +173,33 @@ void LocalConstraint::constrainRotation(Quaternion& rotation, Frame* const)
   translationConstraintDirection(). */
 void WorldConstraint::constrainTranslation(Vec& translation, Frame* const frame)
 {
-  Vec proj;
-  switch (translationConstraintType())
-    {
-    case AxisPlaneConstraint::FREE:
-      break;
-    case AxisPlaneConstraint::PLANE:
-      if (frame->referenceFrame())
+	Vec proj;
+	switch (translationConstraintType())
 	{
-	  proj = frame->referenceFrame()->transformOf(translationConstraintDirection());
-	  translation.projectOnPlane(proj);
+	case AxisPlaneConstraint::FREE:
+		break;
+	case AxisPlaneConstraint::PLANE:
+		if (frame->referenceFrame())
+		{
+			proj = frame->referenceFrame()->transformOf(translationConstraintDirection());
+			translation.projectOnPlane(proj);
+		}
+		else
+			translation.projectOnPlane(translationConstraintDirection());
+		break;
+	case AxisPlaneConstraint::AXIS:
+		if (frame->referenceFrame())
+		{
+			proj = frame->referenceFrame()->transformOf(translationConstraintDirection());
+			translation.projectOnAxis(proj);
+		}
+		else
+			translation.projectOnAxis(translationConstraintDirection());
+		break;
+	case AxisPlaneConstraint::FORBIDDEN:
+		translation = Vec(0.0, 0.0, 0.0);
+		break;
 	}
-      else
-	translation.projectOnPlane(translationConstraintDirection());
-      break;
-    case AxisPlaneConstraint::AXIS:
-      if (frame->referenceFrame())
-	{
-	  proj = frame->referenceFrame()->transformOf(translationConstraintDirection());
-	  translation.projectOnAxis(proj);
-	}
-      else
-	translation.projectOnAxis(translationConstraintDirection());
-      break;
-    case AxisPlaneConstraint::FORBIDDEN:
-      translation = Vec(0.0, 0.0, 0.0);
-      break;
-    }
 }
 
 /*! When rotationConstraintType() is AxisPlaneConstraint::AXIS, constrain \p rotation to be a rotation
@@ -206,24 +207,24 @@ void WorldConstraint::constrainTranslation(Vec& translation, Frame* const frame)
   rotationConstraintDirection(). */
 void WorldConstraint::constrainRotation(Quaternion& rotation, Frame* const frame)
 {
-  switch (rotationConstraintType())
-    {
-    case AxisPlaneConstraint::FREE:
-      break;
-    case AxisPlaneConstraint::PLANE:
-      break;
-    case AxisPlaneConstraint::AXIS:
-      {
-	Vec quat(rotation[0], rotation[1], rotation[2]);
-	Vec axis = frame->transformOf(rotationConstraintDirection());
-	quat.projectOnAxis(axis);
-	rotation = Quaternion(quat, 2.0*acos(rotation[3]));
-	break;
-      }
-    case AxisPlaneConstraint::FORBIDDEN:
-      rotation = Quaternion(); // identity
-      break;
-    }
+	switch (rotationConstraintType())
+	{
+	case AxisPlaneConstraint::FREE:
+		break;
+	case AxisPlaneConstraint::PLANE:
+		break;
+	case AxisPlaneConstraint::AXIS:
+	{
+		Vec quat(rotation[0], rotation[1], rotation[2]);
+		Vec axis = frame->transformOf(rotationConstraintDirection());
+		quat.projectOnAxis(axis);
+		rotation = Quaternion(quat, 2.0*acos(rotation[3]));
+		break;
+	}
+	case AxisPlaneConstraint::FORBIDDEN:
+		rotation = Quaternion(); // identity
+		break;
+	}
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -233,7 +234,7 @@ void WorldConstraint::constrainRotation(Quaternion& rotation, Frame* const frame
 /*! Creates a CameraConstraint, whose constrained directions are defined in the \p camera coordinate
   system. */
 CameraConstraint::CameraConstraint(const Camera* const camera)
-  : AxisPlaneConstraint(), camera_(camera)
+	: AxisPlaneConstraint(), camera_(camera)
 {}
 
 /*! Depending on translationConstraintType(), constrain \p translation to be along an axis or
@@ -241,27 +242,27 @@ CameraConstraint::CameraConstraint(const Camera* const camera)
   translationConstraintDirection(). */
 void CameraConstraint::constrainTranslation(Vec& translation, Frame* const frame)
 {
-  Vec proj;
-  switch (translationConstraintType())
-    {
-    case AxisPlaneConstraint::FREE:
-      break;
-    case AxisPlaneConstraint::PLANE:
-      proj = camera()->frame()->inverseTransformOf(translationConstraintDirection());
-      if (frame->referenceFrame())
-	proj = frame->referenceFrame()->transformOf(proj);
-      translation.projectOnPlane(proj);
-      break;
-    case AxisPlaneConstraint::AXIS:
-      proj = camera()->frame()->inverseTransformOf(translationConstraintDirection());
-      if (frame->referenceFrame())
-	proj = frame->referenceFrame()->transformOf(proj);
-      translation.projectOnAxis(proj);
-      break;
-    case AxisPlaneConstraint::FORBIDDEN:
-      translation = Vec(0.0, 0.0, 0.0);
-      break;
-    }
+	Vec proj;
+	switch (translationConstraintType())
+	{
+	case AxisPlaneConstraint::FREE:
+		break;
+	case AxisPlaneConstraint::PLANE:
+		proj = camera()->frame()->inverseTransformOf(translationConstraintDirection());
+		if (frame->referenceFrame())
+			proj = frame->referenceFrame()->transformOf(proj);
+		translation.projectOnPlane(proj);
+		break;
+	case AxisPlaneConstraint::AXIS:
+		proj = camera()->frame()->inverseTransformOf(translationConstraintDirection());
+		if (frame->referenceFrame())
+			proj = frame->referenceFrame()->transformOf(proj);
+		translation.projectOnAxis(proj);
+		break;
+	case AxisPlaneConstraint::FORBIDDEN:
+		translation = Vec(0.0, 0.0, 0.0);
+		break;
+	}
 }
 
 /*! When rotationConstraintType() is AxisPlaneConstraint::AXIS, constrain \p rotation to be a rotation
@@ -269,22 +270,22 @@ void CameraConstraint::constrainTranslation(Vec& translation, Frame* const frame
   rotationConstraintDirection(). */
 void CameraConstraint::constrainRotation(Quaternion& rotation, Frame* const frame)
 {
-  switch (rotationConstraintType())
-    {
-    case AxisPlaneConstraint::FREE:
-      break;
-    case AxisPlaneConstraint::PLANE:
-      break;
-    case AxisPlaneConstraint::AXIS:
-      {
-	Vec axis = frame->transformOf(camera()->frame()->inverseTransformOf(rotationConstraintDirection()));
-	Vec quat = Vec(rotation[0], rotation[1], rotation[2]);
-	quat.projectOnAxis(axis);
-	rotation = Quaternion(quat, 2.0*acos(rotation[3]));
-      }
-      break;
-    case AxisPlaneConstraint::FORBIDDEN:
-      rotation = Quaternion(); // identity
-      break;
-    }
+	switch (rotationConstraintType())
+	{
+	case AxisPlaneConstraint::FREE:
+		break;
+	case AxisPlaneConstraint::PLANE:
+		break;
+	case AxisPlaneConstraint::AXIS:
+	{
+		Vec axis = frame->transformOf(camera()->frame()->inverseTransformOf(rotationConstraintDirection()));
+		Vec quat = Vec(rotation[0], rotation[1], rotation[2]);
+		quat.projectOnAxis(axis);
+		rotation = Quaternion(quat, 2.0*acos(rotation[3]));
+	}
+		break;
+	case AxisPlaneConstraint::FORBIDDEN:
+		rotation = Quaternion(); // identity
+		break;
+	}
 }
