@@ -1,8 +1,8 @@
 /****************************************************************************
 
- Copyright (C) 2002-2013 Gilles Debunne. All rights reserved.
+ Copyright (C) 2002-2014 Gilles Debunne. All rights reserved.
 
- This file is part of the QGLViewer library version 2.4.0.
+ This file is part of the QGLViewer library version 2.6.4.
 
  http://www.libqglviewer.com - contact@libqglviewer.com
 
@@ -52,11 +52,7 @@ int main(int argc, char** argv)
   // Create a scene, giving a pointer to the associated viewer.
   Scene s(&viewer);
 
-#if QT_VERSION < 0x040000
-  application.setMainWidget(&viewer);
-#else
   viewer.setWindowTitle("callback");
-#endif
 
   help();
 
