@@ -1,8 +1,8 @@
 /****************************************************************************
 
- Copyright (C) 2002-2013 Gilles Debunne. All rights reserved.
+ Copyright (C) 2002-2014 Gilles Debunne. All rights reserved.
 
- This file is part of the QGLViewer library version 2.4.0.
+ This file is part of the QGLViewer library version 2.6.4.
 
  http://www.libqglviewer.com - contact@libqglviewer.com
 
@@ -31,12 +31,7 @@ int main(int argc, char** argv)
   // Instantiate the viewer.
   Viewer viewer;
 
-#if QT_VERSION < 0x040000
-  // Set the viewer as the application main widget.
-  application.setMainWidget(&viewer);
-#else
   viewer.setWindowTitle("cameraLight");
-#endif
 
   // Make the viewer window visible on screen.
   viewer.show();
