@@ -1,8 +1,8 @@
 /****************************************************************************
 
- Copyright (C) 2002-2013 Gilles Debunne. All rights reserved.
+ Copyright (C) 2002-2014 Gilles Debunne. All rights reserved.
 
- This file is part of the QGLViewer library version 2.4.0.
+ This file is part of the QGLViewer library version 2.6.4.
 
  http://www.libqglviewer.com - contact@libqglviewer.com
 
@@ -23,9 +23,7 @@
 #include "viewer.h"
 #include "standardCamera.h"
 
-#if QT_VERSION >= 0x040000
-# include <QKeyEvent>
-#endif
+#include <QKeyEvent>
 
 using namespace std;
 using namespace qglviewer;
@@ -45,21 +43,21 @@ void Viewer::draw()
 
   glBegin(GL_QUAD_STRIP);
   for (int i=0; i<nbSteps; ++i)
-    {
-      const float ratio = i/nbSteps;
-      const float angle = 21.0*ratio;
-      const float c = cos(angle);
-      const float s = sin(angle);
-      const float r1 = 1.0 - 0.8f*ratio;
-      const float r2 = 0.8f - 0.8f*ratio;
-      const float alt = ratio - 0.5f;
-      const float nor = 0.5f;
-      const float up = sqrt(1.0-nor*nor);
-      glColor3f(1.0-ratio, 0.2f , ratio);
-      glNormal3f(nor*c, up, nor*s);
-      glVertex3f(r1*c, alt, r1*s);
-      glVertex3f(r2*c, alt+0.05f, r2*s);
-    }
+	{
+	  const float ratio = i/nbSteps;
+	  const float angle = 21.0*ratio;
+	  const float c = cos(angle);
+	  const float s = sin(angle);
+	  const float r1 = 1.0 - 0.8f*ratio;
+	  const float r2 = 0.8f - 0.8f*ratio;
+	  const float alt = ratio - 0.5f;
+	  const float nor = 0.5f;
+	  const float up = sqrt(1.0-nor*nor);
+	  glColor3f(1.0-ratio, 0.2f , ratio);
+	  glNormal3f(nor*c, up, nor*s);
+	  glVertex3f(r1*c, alt, r1*s);
+	  glVertex3f(r2*c, alt+0.05f, r2*s);
+	}
   glEnd();
 }
 
@@ -72,7 +70,7 @@ void Viewer::init()
   setKeyDescription(Qt::Key_T, "Toggles camera type (perspective or orthographic)");
   setKeyDescription(Qt::Key_M, "Toggles camera mode (standard or QGLViewer)");
 
-  setMouseBindingDescription(Qt::SHIFT + Qt::MidButton, "Change frustum size (for standard camera in orthographic mode)");
+  setMouseBindingDescription(Qt::ShiftModifier, Qt::MidButton, "Change frustum size (for standard camera in orthographic mode)");
 
   // Display help window.
   help();
@@ -90,35 +88,31 @@ void Viewer::keyPressEvent(QKeyEvent *e)
 {
   if (e->key() == Qt::Key_M) {
   // 'M' changes mode : standard or QGLViewer camera
-    ((StandardCamera*)camera())->toggleMode();
-    showMessage();
+	((StandardCamera*)camera())->toggleMode();
+	showMessage();
   } else
-    if (e->key() == Qt::Key_T) {
-    // 'T' changes the projection type : perspective or orthogonal
-      if (camera()->type() == Camera::ORTHOGRAPHIC)
-        camera()->setType(Camera::PERSPECTIVE);
-      else
-        camera()->setType(Camera::ORTHOGRAPHIC);
-        showMessage();
-      } else
-        QGLViewer::keyPressEvent(e);
+	if (e->key() == Qt::Key_T) {
+	// 'T' changes the projection type : perspective or orthogonal
+	  if (camera()->type() == Camera::ORTHOGRAPHIC)
+		camera()->setType(Camera::PERSPECTIVE);
+	  else
+		camera()->setType(Camera::ORTHOGRAPHIC);
+		showMessage();
+	  } else
+		QGLViewer::keyPressEvent(e);
 }
 
 void Viewer::wheelEvent(QWheelEvent *e)
 {
   if ((camera()->type() == Camera::ORTHOGRAPHIC) && (((StandardCamera*)camera())->isStandard()) &&
-#if QT_VERSION >= 0x040000
-    (e->modifiers() & Qt::ShiftModifier))
-#else
-    (e->state() & Qt::ShiftButton))
-#endif
+	(e->modifiers() & Qt::ShiftModifier))
   {
-    ((StandardCamera*)camera())->changeOrthoFrustumSize(e->delta());
-    Q_EMIT cameraChanged();
-    updateGL();
+	((StandardCamera*)camera())->changeOrthoFrustumSize(e->delta());
+	Q_EMIT cameraChanged();
+	updateGL();
   }
   else
-    QGLViewer::wheelEvent(e);
+	QGLViewer::wheelEvent(e);
 }
 
 QString Viewer::helpString() const
@@ -132,7 +126,7 @@ QString Viewer::helpString() const
   text += "On the other hand, the QGLViewer camera fits the near and far distances to the scene radius. ";
   text += "Fine tuning is available using <code>zClippingCoefficient()</code> and <code>zNearCoefficient()</code>. ";
   text += "However, visual results do not seem to be impacted by this zBuffer fitted range.<br><br>";
-  text += "The QGLViewer camera also adapts the orthographic frustum dimensions to the distance to the <code>revolveAroundPoint()</code> to mimic a perspective camera. ";
+  text += "The QGLViewer camera also adapts the orthographic frustum dimensions to the distance to the <code>pivotPoint()</code> to mimic a perspective camera. ";
   text += "Since this behavior may not be needed, this example shows how to override it.<br><br>";
 
   text += "The second viewer displays the first one's camera to show its configuration.<br><br>";
